import { store } from 'https://cdn.skypack.dev/@startinblox/core@0.17';
import { documentReady } from '../utils.js';

/**
 * Initialize rai plugin.
 */
converse.plugins.add('sib-subscribe-to-rai', {
  dependencies: [
    'converse-rai',
  ],
  initialize() {
    const { log } = converse.env;
    const _converse = this._converse;
    const { api } = _converse;

    const getCircles = new Promise((resolve, reject) => {
      const circleInterval = setInterval(async () => {
        let retry = false;
        let circles = [];
        const user = await document.querySelector('sib-auth').getUser();
        const userProfile = await store.getData(user['@id'], this.context);
        for (let circleMembership of await userProfile['circles.ldp:contains']) {
          if (circleMembership) {
            circles.push(await circleMembership['circle.jabberID']);
          } else {
            retry = true;
          }
        }
        if (!retry) {
          clearInterval(circleInterval);
          resolve(circles);
        }
      }, 250);
    });

    const getProjects = new Promise((resolve, reject) => {
      const projectInterval = setInterval(async () => {
        let retry = false;
        let projects = [];
        const user = await document.querySelector('sib-auth').getUser();
        const userProfile = await store.getData(user['@id'], this.context);
        for (let projectMembership of await userProfile['projects.ldp:contains']) {
          if (projectMembership) {
            projects.push(await projectMembership['project.jabberID']);
          } else {
            retry = true;
          }
        }
        if (!retry) {
          clearInterval(projectInterval);
          resolve(projects);
        }
      }, 250);
    });

    const onConnected = async () => {
      // @MattJ Here userRooms is an array of each jabberID the user is on.
      let userRooms = (await Promise.all([
        getCircles,
        getProjects,
      ])).flat();

      log.info(`User rooms: ${userRooms.join(', ')}`);
      await api.rooms.subscribe(userRooms);
    };

    // Send RAI subscriptions when connected to the XMPP endpoint
    api.listen.on('connected', onConnected);
    api.listen.on('reconnected', onConnected);

    // Set timeout so the event is sent once the document is loaded
    // Await document ready state so the event is properly registered
    api.listen.on('chatRoomHasActivity', async jid => {
      await documentReady();
      setTimeout(() => {
        window.dispatchEvent(new CustomEvent('newMessage', {
          detail: {
            jid,
          },
        }));
      }, 250);
    });
  },
});
